
#include <windows.h>
#include <math.h>
#include <stdio.h>

#include "utility.h"
#include "main.h"

#include "quadtree.h"

int display_x = 800;             // Initial screen size
int display_y = 600;

HDC memdc = NULL;	           	 // Memory device context
HBITMAP h_mem_bitmap = NULL;     // Offscreen bitmap



/* World X to screen X */
int screen_x(double x)
{
   return((int)((x/world_max_x) * (display_x-1)));
}
/* End of screen_x */


/* World Y to screen Y */
int screen_y(double y)
{
   return((int)((y/world_max_y) * (display_y-1)));
}
/* End of screen_y */



bool quadtree::render( void )
{
    // See if bounding box is visible

    // Get corner points
    point2d p1(min_x, min_y);
    point2d p2(max_x, min_y);
    point2d p3(max_x, max_y);
    point2d p4(min_x, max_y);
    
    // Entirly to the left of the left frustum line
    if (cross (camera_l_bound, p1 - camera_pos) < 0 &&
        cross (camera_l_bound, p2 - camera_pos) < 0 &&
        cross (camera_l_bound, p3 - camera_pos) < 0 &&
        cross (camera_l_bound, p4 - camera_pos) < 0 ) return(true);
        
        
    // Entirly to the right of the right frustum line
    if (cross (camera_r_bound, p1 - camera_pos) > 0 &&
        cross (camera_r_bound, p2 - camera_pos) > 0 &&
        cross (camera_r_bound, p3 - camera_pos) > 0 &&
        cross (camera_r_bound, p4 - camera_pos) > 0 ) return(true);

    // Otherwise, draw it

    
    bool child_render = false;
    

    double diff_x = camera_pos.x - (min_x + (max_x - min_x) / 2.0);
    double diff_y = camera_pos.y - (min_y + (max_y - min_y) / 2.0);
  
    double dist = diff_x * diff_x + diff_y * diff_y;

    double cut = (world_max_x*world_max_x + world_max_y*world_max_y)/pow(dist_res, max_levels - level );

    
    
    // If it's not a leaf is must have children
    if (!is_leaf && dist < cut )
    {
        if (nodes[0]->render()) child_render = true;
        if (nodes[1]->render()) child_render = true;
        if (nodes[2]->render()) child_render = true;
        if (nodes[3]->render()) child_render = true;
    }


    if (child_render) return(true);    
    
    
    int x1 = screen_x(min_x);
    int y1 = screen_y(min_y);
    
    int x2 = screen_x(max_x);
    int y2 = screen_y(max_y);



    MoveToEx(memdc, x1, y1, NULL);
    LineTo(memdc, x2, y1);
    LineTo(memdc, x2, y2);
    LineTo(memdc, x1, y2);
    LineTo(memdc, x1, y1);


return(true);
}


void draw_camera( void )
{
    
    int x =  screen_x(camera_pos.x);    
    int y =  screen_y(camera_pos.y);    
    
    Ellipse(memdc, x-10, y-10, x+10, y+10);
    
    int nx = screen_x(camera_pos.x + 10 * camera_dir.x);
    int ny = screen_y(camera_pos.y + 10 * camera_dir.y);
     
    MoveToEx(memdc, x, y, 0);
    LineTo(memdc, nx, ny);
        
    nx = screen_x(camera_pos.x + 10 * camera_l_bound.x);
    ny = screen_y(camera_pos.y + 10 * camera_l_bound.y);    
    MoveToEx(memdc, x, y, 0);
    LineTo(memdc, nx, ny);
    
    nx = screen_x(camera_pos.x + 10 *  camera_r_bound.x);
    ny = screen_y(camera_pos.y + 10 *  camera_r_bound.y);
    
    MoveToEx(memdc, x, y, 0);
    LineTo(memdc, nx, ny);
    
}



/* Draw game display */
void draw( void )
{
    RECT r;
    
    // Erase in black
    SelectObject(memdc, (HBRUSH) GetStockObject(WHITE_BRUSH));
    SelectObject(memdc, (HBRUSH) GetStockObject(BLACK_PEN));    
   
    Rectangle(memdc, 0, 0, display_x, display_y);    
    
    main_tree->render();
  
    draw_camera();
    
    char buff[300];
   
    sprintf(buff, "FPS: %d Total Regions: %d", fps, bound_total);

    TextOut(memdc, 2, 2, buff, strlen(buff));

}
/* End of draw_game */



void graphics_paint (HWND hwnd)
{
    PAINTSTRUCT ps;
    HDC hdc = BeginPaint(hwnd, &ps);
    BitBlt(hdc, 0, 0, display_x, display_y, memdc, 0, 0, SRCCOPY);
    EndPaint(hwnd, &ps);
}



/* Create off screen buffer */
void setup_display( HWND hwnd )
{
    // Get main DC
    HDC hdc = GetDC(hwnd);

  	// Create Memory device contextand bitmap
    memdc = CreateCompatibleDC(hdc);	

    // Create off screen buffer
  	h_mem_bitmap = CreateCompatibleBitmap(hdc, display_x, display_y);

  	// Select the off screen buffer into the DC
   	SelectObject(memdc, h_mem_bitmap);		

    ReleaseDC(hwnd, hdc);       // Main DC is no longer needed
}
/* End of setup_display */


/* Resize the display and update buffers */
void resize_display(HWND hwnd, LPARAM l)
{
    display_x = LOWORD(l);
    display_y = HIWORD(l);

    DeleteObject(h_mem_bitmap);		// Delete old screen buffer
    DeleteDC(memdc);		      	// Delete old memory DC

    setup_display(hwnd);

    InvalidateRect(hwnd, NULL, true);   
}
/* End of resize_display */



void destroy_display( void )
{
    DeleteObject(h_mem_bitmap);		// Delete off screen buffer
    DeleteDC(memdc);	       		// Delete memory DC
}

