/*
    Spring-Mass Simulation
    (C) 2007 Jason Hunt
    nulluser@gmail.com
*/

#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "utility.h"
#include "quadtree.h"

#include "graphics.h"


#define SECOND_TIMER    1           // ID for frame timer
#define UPDATE_TIMER    2           // ID for frame timer

#define FRAME_TIME      10          // Millisecond delay for frames




unsigned int fps = 0;
unsigned int updates = 0;


double inital_mouse_x;
double inital_mouse_y;
double mouse_time;

bool mouse_down = false;



void check_keys ( void )
{
    double speed = 0.2;


    if(GetAsyncKeyState(0x57)) // W key Forward
    {
        camera_pos.x += speed * camera_dir.x;
        camera_pos.y += speed * camera_dir.y;
    }
    
    if(GetAsyncKeyState(0x53)) // W key Forward
    {
        camera_pos.x -= speed * camera_dir.x;
        camera_pos.y -= speed * camera_dir.y;
    }   

    if(GetAsyncKeyState(0x41)) // A - Strafe left
    {
        camera_ang -= speed / 10.0;

        camera_dir.x = cos(camera_ang);
        camera_dir.y = sin(camera_ang);
    }
        

    if(GetAsyncKeyState(0x44)) // D - Strafe right
    {
        camera_ang += speed / 10.0;

        camera_dir.x = cos(camera_ang);
        camera_dir.y = sin(camera_ang);
    
        
    }
    
    if (camera_pos.x < 0) camera_pos.x = 0;
    if (camera_pos.x > world_max_x) camera_pos.x = world_max_x;
    
    if (camera_pos.y < 0) camera_pos.y = 0;
    if (camera_pos.y > world_max_y) camera_pos.y = world_max_y;
}



/* The window Function */
LRESULT CALLBACK WinProc(HWND hwnd, UINT msg, WPARAM w, LPARAM l)
{


    switch(msg)
    {
        case WM_CREATE: 
        {
            srand(GetTickCount());
            setup_display(hwnd);
                  
            setup_quadtree(); 
 
            SetTimer(hwnd, UPDATE_TIMER, FRAME_TIME, NULL);
            SetTimer(hwnd, SECOND_TIMER, 1000, NULL);
            return(0);
        }

        case WM_CHAR:
        {
            if (w == 27) DestroyWindow(hwnd);   // Escape exits
            return(0);
        }

        case WM_PAINT:
            graphics_paint(hwnd);
            return(0);

        case WM_TIMER:            
            if (w == UPDATE_TIMER)
            {
                check_keys();

                update_camera();

                draw();
                updates++;
                
                // Force redraw 
                InvalidateRect(hwnd, NULL, 0);                
            }
            if (w == SECOND_TIMER)
            {               
                fps = updates;
                updates = 0;
            }
            return(0);  
            
        case WM_LBUTTONDOWN:
        {
            //reset_ball();
        }
            
        return(0);
    
        case WM_LBUTTONUP:
        {                
            delete main_tree;
            setup_quadtree(); 
        }
        return(0);       
                 
        case WM_RBUTTONDOWN:
            
            return(0);

        case WM_DESTROY:   
            destroy_display();
        
            PostQuitMessage(0);
            return(0);
            
        case WM_MOUSEMOVE:
            return(0);
            
            
         case WM_KILLFOCUS:
            mouse_down = 0;
            return(0);            
            
            
        case WM_SIZE:
            resize_display(hwnd, l);
            return(0);

        default:
            return(DefWindowProc(hwnd, msg, w, l));
    }

    return(0);  
}
/* End of WinProc */


/* WinMain */
int WINAPI WinMain(HINSTANCE h_this_inst, HINSTANCE h_prev_inst, LPSTR args, int mode)
{
    debug_start();

    WNDCLASS wnd_class;

    char program_name[] = "QTREE";
        
    wnd_class.hInstance = h_this_inst;
    wnd_class.lpszClassName = program_name;
    wnd_class.lpfnWndProc = WinProc;
    wnd_class.style = CS_OWNDC;//0;
    
    wnd_class.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wnd_class.hCursor = LoadCursor(NULL, IDC_ARROW);
    wnd_class.lpszMenuName = NULL;
    
    wnd_class.cbClsExtra = 0;
    wnd_class.cbWndExtra = 0;
    
    wnd_class.hbrBackground = NULL; // Not needed
    
    if (!RegisterClass(&wnd_class)) return(0);
        
    // Create the window
    HWND hwnd = CreateWindow(program_name, "Quadtree", 
                             WS_OVERLAPPEDWINDOW,
                             300, 100,                                      // Pos
                             display_x, display_y,              // Size
                             HWND_DESKTOP, NULL, h_this_inst, NULL);

    ShowWindow(hwnd, mode);
    UpdateWindow(hwnd);
           
    MSG msg;
    
    // Windows message loop
    while(GetMessage(&msg, NULL, 0, 0))
    { 
        TranslateMessage(&msg);
        DispatchMessage(&msg);                            
    }
                        
    return(msg.wParam);
}
/* End of winmain */



