#include <windows.h>
#include <stdio.h>

#include "math.h"
#include "utility.h"
#include "quadtree.h"
#include "graphics.h"


double world_max_x = 100.0;      // Size of world
double world_max_y = 100.0;


point2d camera_pos(world_max_x/2.0, world_max_x/2.0);
point2d camera_dir(1,0);

point2d camera_l_bound(1,0);
point2d camera_r_bound(1,0);

double camera_ang = 0;

double dist_res = 2.0;


unsigned int bound_total = 0;



//unsigned int num_triangles = 10000;  // NUmber of triangles for test

unsigned int max_levels = 5;       // Maximun number of tree levels

//unsigned int max_tri_node = 1;      // Maximun number of triangles in a node before
                                    // it will try to subdivide
                                
quadtree *main_tree = NULL;         // Main tree for testing
    
bool debug_tree = false;            // True if we will dump tree on load


/* Setup a test quadtree */
void setup_quadtree( void )
{
    // Create terrain quadtree 
    main_tree = new quadtree(0,0, world_max_x, world_max_y, max_levels);

    if (debug_tree)
        main_tree->dump(0);   // Dump main tree for debugging
}
/* End of setup quadtree */



/* Creates a quadtree from a list of triangles and a bounding dimension */
quadtree::quadtree(double nmin_x, double nmin_y,  double nmax_x, double nmax_y, int l)
{
    min_x = nmin_x;   max_x = nmax_x;
    min_y = nmin_y;   max_y = nmax_y;    

    is_leaf = true;      // Default to a leaf with no nodes

    bound_total += 1;

    if (l <= 0) return; // Too many levels

    level = l;

    // Compute center of bounding region
    double mid_x = min_x + (max_x - min_x) / 2.0;
    double mid_y = min_y + (max_y - min_y) / 2.0;

    nodes[0] = new quadtree(min_x, min_y, mid_x, mid_y, l-1);
    nodes[1] = new quadtree(mid_x, min_y, max_x, mid_y, l-1);
    nodes[2] = new quadtree(min_x, mid_y, mid_x, max_y, l-1);
    nodes[3] = new quadtree(mid_x, mid_y, max_x, max_y, l-1);    

    is_leaf = false;    // Node is not a leaf
}
/* End of quadtree constructor */



quadtree::~quadtree()
{    
    if (!is_leaf)
    {
        delete nodes[0];
        delete nodes[1];
        delete nodes[2];
        delete nodes[3];
    }
    
}





/* Dump a quadtree to the debug file */
void quadtree::dump (int level) 
{
    char space[50] = {0}; // Spacing
  
    // Create spacing
    for (int i = 0; i < level; i++)
        strcat(space, "   ");

    char b[400]; // output buffer

    sprintf(b, "%s bounds (%f, %f) (%f, %f) leaf: %d", space, 
            min_x, min_y, max_x, max_y, is_leaf);

    debug(b);



    // Dump its children if it's not a leaf
    if (!is_leaf)
    {
        nodes[0]->dump(level+1);
        nodes[1]->dump(level+1);
        nodes[2]->dump(level+1);
        nodes[3]->dump(level+1);
    }
}
/* End of dump */






void update_camera( void )
{
    camera_dir.x = cos(camera_ang);
    camera_dir.y = sin(camera_ang);   

    camera_l_bound.x = cos(camera_ang+0.5);
    camera_l_bound.y = sin(camera_ang+0.5);

    camera_r_bound.x = cos(camera_ang-0.5);
    camera_r_bound.y = sin(camera_ang-0.5);
    
}





