
#include <windows.h>
#include <math.h>

#include "utility.h"

#include <stdio.h>

#define epsilon 0.00001

double length( point2d p )
{
    return sqrt(p.x * p.x + p.y * p.y);
}


void normalize(point2d p)
{
    double l = length(p);
    
    if (l < epsilon) l = epsilon;
    
    p = p / l;
}



point2d get_ortho_unit(point2d p)
{
    point2d r;
    r.x = -p.y;
    r.y = p.x;
    
    normalize(r);
    return(r);
}


// Returns direction of cross product
double cross(point2d p1, point2d p2)
{
    return p2.x * p1.y - p1.x * p2.y;    
}



double dot ( point2d p1, point2d p2)
{
return (p1.x * p2.x + p1.y * p2.y)  ;  
    
}

/* Turn two cords into a point */
 point2d get_point( double x, double y)
{
    point2d p;
    p.x = x; p.y=y;
    return(p);
}
/* End of get point */


/* - operator for points */
point2d operator/(point2d p, double s)
{
    point2d r;

    if (fabs(s) < 0.00001) return(get_point(0,0));
    r.x = p.x/s;
    r.y = p.y/s;
    return(r);
}

/* - operator for points */
point2d operator* ( double s, point2d p)
{
    point2d r;
    r.x = s * p.x;
    r.y = s * p.y;
    return(r);
}

/* - operator for points */
point2d operator* ( point2d p, double s)
{
    return(s * p);
}

/* - operator for points */
point2d operator- ( point2d p1, point2d p2)
{
    point2d r;
    r.x = p1.x - p2.x;  r.y = p1.y - p2.y;
    return(r);
}

/* - operator for points */
point2d operator+ ( point2d p1, point2d p2)
{
    point2d r;
    r.x = p1.x + p2.x;   r.y = p1.y + p2.y;
    return(r);
}

/* - operator for points */
point2d operator+= ( point2d &p1, point2d p2)
{
    p1.x += p2.x; p1.y += p2.y;
    return(p1);
}

/* - operator for points */
point2d operator-= ( point2d &p1, point2d p2)
{
    p1.x -= p2.x;
    p1.y -= p2.y;
    return(p1);
}


/* Return a random number in the specified range */
double f_rand(double s, double e)
{
    return((e - s)*(rand() / (double)RAND_MAX) + s);
}
/* End of f_rand */


/* Return current time in seconds */
double get_time( void )
{
  //  EnterCriticalSection(&time_lock);           

    static LARGE_INTEGER freq;
    bool init = 0;

    if (!init)
    {
        QueryPerformanceFrequency(&freq);
        init = 1;
    }

    LARGE_INTEGER c;

    QueryPerformanceCounter(&c);

    double value = c.QuadPart / (double)freq.QuadPart;
    
   // LeaveCriticalSection(&time_lock);
    
    return(value);
}
/* End of get_time */





void write_console(char * str)
{
   
    static HANDLE h_console = INVALID_HANDLE_VALUE;

    if (h_console == INVALID_HANDLE_VALUE)
    {
        if (!AllocConsole()) return;
        h_console = GetStdHandle(STD_OUTPUT_HANDLE);
        
        if (h_console == INVALID_HANDLE_VALUE) return;        
    }
            
    DWORD num_out;
            
    WriteConsole(h_console, str, strlen(str), &num_out, NULL);
}




void debug_start( void )
{
    FILE *f = fopen("debug.txt", "wt");        
    fclose(f);    
}



void debug(char *t)
{
    FILE *f = fopen("debug.txt", "at");        
    fprintf(f, "%s\n", t);
    fclose(f);    
}


void debug(char *t, double n)
{
    FILE *f = fopen("debug.txt", "at");        
    fprintf(f, "%s: %f\n", t, n);
    fclose(f);    
}

void debug(char *t, int n)
{
    FILE *f = fopen("debug.txt", "at");        
    fprintf(f, "%s: %d\n", t, n);
    fclose(f);    
}



COLORREF rand_color( void )
{
    switch (rand() % 8)
    {
        case 0: return(RGB(128, 128, 128));
        case 1: return(RGB(128, 0, 0));
        case 2: return(RGB(0, 128,0));
        case 3: return(RGB(0, 0, 128));
        case 4: return(RGB(0, 0, 0));
        case 5: return(RGB(255, 0, 0));
        case 6: return(RGB(0, 255,0));
        case 7: return(RGB(0, 0, 255));
    }     
    
}

